@@ -1,7 +1,8 @@
 /* directory.c
  *
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
- * Copyright  2005 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (c) 2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright (c) 2005 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (c) 2007 Hubert Figuiere <hub@figuiere.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,23 @@
 #include <sys/types.h>
 #include <utime.h>
 #include <unistd.h>
+#ifdef HAVE_SYS_STATVFS_H
+# include <sys/statvfs.h>
+#endif
+#ifdef HAVE_SYS_PARAM_H
+# include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_VFS_H
+# include <sys/vfs.h>
+#endif
+#ifdef HAVE_SYS_MOUNT_H
+# include <sys/mount.h>
+#endif
+#ifdef HAVE_SYS_STATFS_H
+# include <sys/statfs.h>
+#endif
+#include <fcntl.h>
+
 
 #ifdef HAVE_LIBEXIF
 #include <libexif/exif-data.h>
@@ -62,21 +80,45 @@ static const struct {
 } mime_table[] = {
 	{"jpeg", GP_MIME_JPEG},
 	{"jpg",  GP_MIME_JPEG}, 
+	{"thm",  GP_MIME_JPEG}, 
 	{"tif",  GP_MIME_TIFF},
 	{"ppm",  GP_MIME_PPM},
 	{"pgm",  GP_MIME_PGM},
 	{"avi",  GP_MIME_AVI},
 	{"mov",  GP_MIME_QUICKTIME},
+	{"moov", GP_MIME_QUICKTIME},
+	{"qt",   GP_MIME_QUICKTIME},
+	{"qtvr", GP_MIME_QUICKTIME},
+	{"mp2",  GP_MIME_MPEG},
 	{"mpg",  GP_MIME_MPEG},
+	{"mpeg", GP_MIME_MPEG},
+	{"mpe",  GP_MIME_MPEG},
+	{"mts",  GP_MIME_AVCHD},
+	{"m2ts", GP_MIME_AVCHD},
 	{"pbm", "image/x-portable-bitmap"},
 	{"crw",  GP_MIME_CRW},
+	{"rw2",  GP_MIME_RW2},
 	{"cr2",  GP_MIME_RAW},
 	{"nef",  GP_MIME_RAW},
 	{"mrw",  GP_MIME_RAW},
 	{"dng",  GP_MIME_RAW},
 	{"sr2",  GP_MIME_RAW},
+	{"raf",  GP_MIME_RAW},
+	{"erf",  GP_MIME_RAW},
+	{"orf",  GP_MIME_RAW},
+	{"arw",  GP_MIME_RAW},
+	{"pef",  GP_MIME_RAW},
+	{"raw",  GP_MIME_RAW},
+	{"dcr",  GP_MIME_RAW},
+	{"x3f",  GP_MIME_RAW},
+	{"srw",  GP_MIME_RAW},
+	{"gf1",  GP_MIME_RAW},
+	{"srw",  GP_MIME_RAW},
 	{"png",  GP_MIME_PNG},
 	{"wav",  GP_MIME_WAV},
+	{"3gp",  "video/3gpp"},
+	{"3g2",  "video/x-3gpp2"},
+	{"dif",  "video/dv"},
 	{NULL, NULL}
 };
 
@@ -149,6 +191,27 @@ int camera_abilities (CameraAbilitiesList *list)
         return (GP_OK);
 }
 
+static int
+_get_path (GPPort *port, const char *folder, const char *file, char *path, unsigned int size) {
+	if (port->type == GP_PORT_DISK) {
+		GPPortInfo	info;
+		int ret;
+		char *xpath;
+
+		ret = gp_port_get_info (port, &info);
+		if (ret < GP_OK)
+			return ret;
+		xpath = strchr (info.path,':');
+		if (!xpath) xpath = info.path; else xpath++;
+		snprintf (path, size, "%s/%s/%s", xpath, folder, file);
+	} else {
+		/* old style access */
+		snprintf (path, size, "%s/%s", folder, file);
+	}
+	return GP_OK;
+}
+
+
 static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
@@ -160,25 +223,29 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	Camera *camera = (Camera*)data;
 
 	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
+		char		*path;
+		GPPortInfo	info;
+		int		ret;
 
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (f, sizeof(f), "%s/%s/",
-			settings.disk.mountpoint, 
-			folder
-		);
+		ret = gp_port_get_info (camera->port, &info);
+		if (ret < GP_OK)
+			return ret;
+		path = strchr (info.path,':');
+		if (!path) path = info.path; else path++;
+		snprintf (f, sizeof(f), "%s/%s/", path, folder);
+		gp_log (GP_LOG_DEBUG, "directory/file_list_func", "%s", f);
 		/* UNIX / is empty, or we recurse through the whole fs */
-		if (	(!strcmp(settings.disk.mountpoint, "") ||
-			 !strcmp(settings.disk.mountpoint, "/")) &&
+		if (	(!strcmp(path, "") || !strcmp(path, "/")) &&
 			!strcmp(folder,"/")
 		)
 			return GP_OK;
 	} else {
 		/* old style access */
-		if (folder[strlen(folder)-1] != '/')
-			sprintf (f, "%s%c", folder, '/');
-		else
-			strcpy (f, folder);
+		if (folder[strlen(folder)-1] != '/') {
+			snprintf (f, sizeof(f), "%s%c", folder, '/');
+		} else {
+			strncpy (f, folder, sizeof(f));
+		}
 	}
 	dir = gp_system_opendir ((char*) f);
 	if (!dir)
@@ -199,7 +266,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 				"'%s'..."), f);
 	n = 0;
 	while ((de = gp_system_readdir(dir))) {
-
+		const char * filename = NULL;
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
@@ -208,12 +275,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			return (GP_ERROR_CANCEL);
 		}
 
-		if (strcmp(gp_system_filename(de), "." ) &&
-		    strcmp(gp_system_filename(de), "..")) {
-			sprintf (buf, "%s%s", f, gp_system_filename (de));
-			if (gp_system_is_file (buf) && (get_mime_type (buf)))
-				gp_list_append (list, gp_system_filename (de),
-						NULL);
+		filename = gp_system_filename(de);
+		if (*filename != '.') {
+			snprintf (buf, sizeof(buf), "%s%s", f, filename);
+			if (gp_system_is_file (buf) && (get_mime_type (buf))) {
+				gp_list_append (list, filename,	NULL);
+			}
 		}
 		n++;
 	}
@@ -235,30 +302,35 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	Camera *camera = (Camera*)data;
 
 	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
+		char		*path;
+		GPPortInfo	info;
+		int		ret;
 
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (f, sizeof(f), "%s/%s/",
-			settings.disk.mountpoint, 
-			folder
-		);
+		ret = gp_port_get_info (camera->port, &info);
+		if (ret < GP_OK)
+			return ret;
+		path = strchr (info.path,':');
+		if (!path) path = info.path; else path++;
+
+		snprintf (f, sizeof(f), "%s/%s/", path, folder);
+		gp_log (GP_LOG_DEBUG, "directory/folder_list_func", "%s", f);
 		/* UNIX / is empty, or we recurse through the whole fs */
-		if (	(!strcmp(settings.disk.mountpoint, "") ||
-			 !strcmp(settings.disk.mountpoint, "/")) &&
+		if (	(!strcmp(path, "") || !strcmp(path, "/")) &&
 			!strcmp(folder,"/")
 		)
 			return GP_OK;
 	} else {
 		/* old style access */
 		/* Make sure we have 1 delimiter */
-		if (folder[strlen(folder)-1] != '/')
-			sprintf (f, "%s%c", folder, '/');
-		else
-			strcpy (f, folder);
+		if (folder[strlen(folder)-1] != '/') {
+			snprintf (f, sizeof(f), "%s%c", folder, '/');
+		} else {
+			strncpy (f, folder, sizeof(f));
+		}
 	}
 	dir = gp_system_opendir ((char*) f);
 	if (!dir)
-		return (GP_ERROR);
+		return GP_ERROR;
 	/* Count the files */
 	n = 0;
 	while (gp_system_readdir (dir))
@@ -267,32 +339,31 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 	dir = gp_system_opendir (f);
 	if (!dir)
-		return (GP_ERROR);
-	id = gp_context_progress_start (context, n, _("Listing folders in "
-					"'%s'..."), folder);
+		return GP_ERROR;
+	id = gp_context_progress_start (context, n, _("Listing folders in '%s'..."), folder);
 	n = 0;
 	while ((de = gp_system_readdir (dir))) {
+		const char * filename = NULL;
+
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
 		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
 			gp_system_closedir (dir);
-			return (GP_ERROR_CANCEL);
+			return GP_ERROR_CANCEL;
 		}
-		if ((strcmp (gp_system_filename (de), "." ) != 0) &&
-		    (strcmp (gp_system_filename (de), "..") != 0)) {
-			sprintf (buf, "%s%s", f, gp_system_filename (de));
-
+		filename = gp_system_filename (de);
+		if (*filename != '.') {
+			snprintf (buf, sizeof(buf), "%s%s", f, filename);
+			
 			/* lstat ... do not follow symlinks */
 			if (lstat (buf, &st) != 0) {
-				gp_context_error (context, _("Could not get information "
-						  "about '%s' (%m)."), buf);
-				return (GP_ERROR);
+				gp_context_error (context, _("Could not get information about '%s' (%m)."), buf);
+				return GP_ERROR;
+			}
+			if (S_ISDIR (st.st_mode)) {
+				gp_list_append(list,	filename,	NULL);
 			}
-			if (S_ISDIR (st.st_mode))
-				gp_list_append (list,
-						gp_system_filename (de),
-						NULL);
 		}
 		n++;
 	}
@@ -309,20 +380,12 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 	const char *mime_type;
 	struct stat st;
 	Camera *camera = (Camera*)data;
+	int result;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			file
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, file);
-	}
+	gp_log (GP_LOG_DEBUG, "directory/get_info_func", "%s %s", folder, file);
+	result = _get_path (camera->port, folder, file, path, sizeof(path));
+	if (result < GP_OK)
+		return result;
 
 	if (lstat (path, &st) != 0) {
 		gp_context_error (context, _("Could not get information "
@@ -356,22 +419,12 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 	       CameraFileInfo info, void *data, GPContext *context)
 {
 	int retval;
-	char path_old[1024], path_new[1024], path[1024];
+	char path[1024];
 	Camera *camera = (Camera*)data;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			file
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, file);
-	}
+	retval = _get_path (camera->port, folder, file, path, sizeof(path));
+	if (retval < GP_OK)
+		return retval;
 
 	/* We don't support updating permissions (yet) */
 	if (info.file.fields & GP_FILE_INFO_PERMISSIONS)
@@ -390,8 +443,9 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 		}
 	}
 
+#if 0 /* implement this using new api -Marcus */
 	if (info.file.fields & GP_FILE_INFO_NAME) {
-        	if (!strcmp (info.file.name, file))
+        	if (!strcasecmp (info.file.name, file))
         	        return (GP_OK);
 
 	        /* We really have to rename the poor file... */
@@ -426,6 +480,7 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 	                }
 	        }
 	}
+#endif
 
         return (GP_OK);
 }
@@ -437,39 +492,39 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
         char path[1024];
 	int result = GP_OK;
-#ifdef DEBUG
-	unsigned int i, id;
-#endif
+	struct stat stbuf;
+	int fd, id;
+	unsigned int curread, toread;
+	unsigned char *buf;
 #ifdef HAVE_LIBEXIF
 	ExifData *data;
-	unsigned char *buf;
 	unsigned int buf_len;
 #endif /* HAVE_LIBEXIF */
 	Camera *camera = (Camera*)user_data;
+	struct stat st;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
+	result = _get_path (camera->port, folder, filename, path, sizeof(path));
+	gp_log (GP_LOG_DEBUG, "directory/get_file_func", "%s %s",folder,filename);
+	if (result < GP_OK)
+		return result;
+	gp_log (GP_LOG_DEBUG, "directory/get_file_func", "->%s",path);
 
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			filename
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, filename);
+	if (lstat (path, &st) != 0) {
+		gp_context_error (context, _("Could not get information "
+			"about '%s' in '%s' (%m)."), filename, folder);
+		return (GP_ERROR);
 	}
+	gp_file_set_mtime (file, st.st_mtime);
 
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
-		result = gp_file_open (file, path);
-		break;
 #ifdef DEBUG
 	case GP_FILE_TYPE_PREVIEW:
-		result = gp_file_open (file, path);
-		break;
 #endif
+		fd = open (path,O_RDONLY);
+		if (fd == -1)
+			return GP_ERROR_IO_READ;
+		break;
 #ifdef HAVE_LIBEXIF
 	case GP_FILE_TYPE_EXIF:
 		data = exif_data_new_from_file (path);
@@ -486,22 +541,52 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	default:
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
-	if (result < 0)
-		return (result);
 
-#ifdef DEBUG
-	id = gp_context_progress_start (context, 500., "Getting file...");
+	if (-1 == fstat(fd,&stbuf)) {
+		close (fd);
+		return GP_ERROR_IO_READ;
+	}
+#define BLOCKSIZE 65536
+	/* do it in 64kb blocks */
+	buf = malloc(BLOCKSIZE);
+	if (!buf) {
+		close (fd);
+		return GP_ERROR_NO_MEMORY;
+	}
+
+	curread = 0;
+	id = gp_context_progress_start (context, (1.0*stbuf.st_size/BLOCKSIZE), _("Getting file..."));
 	GP_DEBUG ("Progress id: %i", id);
-	for (i = 0; i < 500; i++) {
-		gp_context_progress_update (context, id, i + 1);
+	result = GP_OK;
+	while (curread < stbuf.st_size) {
+		int ret;
+
+		toread = stbuf.st_size-curread;
+		if (toread>BLOCKSIZE) toread = BLOCKSIZE;
+		ret = read(fd,buf,toread);
+		if (ret == -1) {
+			result = GP_ERROR_IO_READ;
+			break;
+		}
+		curread += ret;
+		gp_file_append (file, buf, ret);
+		gp_context_progress_update (context, id, (1.0*curread/BLOCKSIZE));
 		gp_context_idle (context);
-		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL)
-			return (GP_ERROR_CANCEL);
-		usleep (10);
+		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
+			result = GP_ERROR_CANCEL;
+			break;
+		}
+#if 0
+		/* We could take 2 seconds to download this image. everytime. */
+		/* But actually this driver is used in production by some frontends,
+		 * so do not delay at all
+		 */
+		usleep(2000000/(stbuf.st_size/BLOCKSIZE));
+#endif
 	}
 	gp_context_progress_stop (context, id);
-#endif
-
+	free (buf);
+	close (fd);
 	return (GP_OK);
 }
 
@@ -529,21 +614,12 @@ make_dir_func (CameraFilesystem *fs, const char *folder, const char *name,
 {
 	char path[2048];
 	Camera *camera = (Camera*)data;
+	int result;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			name
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, name);
-	}
-	return (gp_system_mkdir (path));
+	result = _get_path (camera->port, folder, name, path, sizeof(path));
+	if (result < GP_OK)
+		return result;
+	return gp_system_mkdir (path);
 }
 
 static int
@@ -552,21 +628,12 @@ remove_dir_func (CameraFilesystem *fs, const char *folder, const char *name,
 {
 	char path[2048];
 	Camera *camera = (Camera*)data;
+	int result;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			name
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, name);
-	}
-	return (gp_system_rmdir (path));
+	result = _get_path (camera->port, folder, name, path, sizeof(path));
+	if (result < GP_OK)
+		return result;
+	return gp_system_rmdir (path);
 }
 
 static int
@@ -577,19 +644,9 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	int result;
 	Camera *camera = (Camera*)data;
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			file
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, file);
-	}
+	result = _get_path (camera->port, folder, file, path, sizeof(path));
+	if (result < GP_OK)
+		return result;
 	result = unlink (path);
 	if (result) {
 		gp_context_error (context, _("Could not delete file '%s' "
@@ -615,19 +672,9 @@ put_file_func (CameraFilesystem *fs, const char *folder,
 
 	gp_file_get_name (file, &name);
 
-	if (camera->port->type == GP_PORT_DISK) {
-		GPPortSettings settings;
-
-		gp_port_get_settings (camera->port, &settings);
-		snprintf (path, sizeof(path), "%s/%s/%s",
-			settings.disk.mountpoint, 
-			folder,
-			name
-		);
-	} else {
-		/* old style access */
-		snprintf (path, sizeof (path), "%s/%s", folder, name);
-	}
+	result = _get_path (camera->port, folder, name, path, sizeof(path));
+	if (result < GP_OK)
+		return result;
 
 	result = gp_file_save (file, path);
 	if (result < 0)
@@ -674,6 +721,62 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 }
 #endif
 
+static int
+storage_info_func (CameraFilesystem *fs,
+                CameraStorageInformation **sinfos,
+                int *nrofsinfos,
+                void *data, GPContext *context
+) {
+	Camera 				*camera = data;
+	CameraStorageInformation	*sfs;
+
+#if defined(linux) && defined(HAVE_STATFS)
+	struct	statfs		stfs;
+	char		*xpath;
+	int		ret;
+	GPPortInfo	info;
+
+	ret = gp_port_get_info (camera->port, &info);
+	if (ret < GP_OK)
+		return ret;
+	xpath = strchr (info.path,':');
+	if (!xpath) xpath = info.path; else xpath++;
+
+	if (-1 == statfs (xpath, &stfs))
+		return GP_ERROR_NOT_SUPPORTED;
+
+	sfs = malloc (sizeof (CameraStorageInformation));
+	if (!sfs)
+		return GP_ERROR_NO_MEMORY;
+	*sinfos = sfs;
+	*nrofsinfos = 1;
+
+	sfs->fields =	GP_STORAGEINFO_BASE		|
+			GP_STORAGEINFO_DESCRIPTION	|
+			GP_STORAGEINFO_STORAGETYPE	|
+			GP_STORAGEINFO_FILESYSTEMTYPE	|
+			GP_STORAGEINFO_ACCESS		|
+			GP_STORAGEINFO_MAXCAPACITY	|
+			GP_STORAGEINFO_FREESPACEKBYTES;
+	;
+	strcpy (sfs->basedir, "/");
+	strcpy (sfs->description, "Directory Driver");
+	sfs->type		= GP_STORAGEINFO_ST_REMOVABLE_RAM;
+	sfs->fstype		= GP_STORAGEINFO_FST_GENERICHIERARCHICAL;
+	sfs->access		= GP_STORAGEINFO_AC_READWRITE;
+	if (stfs.f_bsize >= 1024) {
+		sfs->capacitykbytes	= stfs.f_blocks*(stfs.f_bsize/1024);
+		sfs->freekbytes		= stfs.f_bavail*(stfs.f_bsize/1024);
+	} else {
+		sfs->capacitykbytes	= stfs.f_blocks/(1024/stfs.f_bsize);
+		sfs->freekbytes		= stfs.f_bavail/(1024/stfs.f_bsize);
+	}
+	return GP_OK;
+#endif
+
+	return GP_ERROR_NOT_SUPPORTED;
+} 
+
 static CameraFilesystemFuncs fsfuncs = {
 	.file_list_func = file_list_func,
 	.folder_list_func = folder_list_func,
@@ -684,6 +787,7 @@ static CameraFilesystemFuncs fsfuncs = {
 	.del_file_func = delete_file_func,
 	.make_dir_func = make_dir_func,
 	.remove_dir_func = remove_dir_func,
+	.storage_info_func = storage_info_func,
 };
 
 int
