@@ -37,9 +37,9 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <gphoto2-library.h>
-#include <gphoto2-result.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-result.h>
+#include <gphoto2/gphoto2-port-log.h>
 #include <exif.h>
 
 #ifdef ENABLE_NLS
@@ -125,15 +125,13 @@
 #define PLAY_DATE_DISP_PTR	0x20
 #define DATE_FORMAT_PTR		0x21
 #define TIMESTAMP_PTR		0x22
-extern unsigned char k_calculate_checksum (unsigned char *buf,
-		unsigned long int len);
 
 /** Local functions **********************************************************/
 
 /* 
  * Check the integrity of datas
  */
-unsigned char
+static unsigned char
 k_calculate_checksum (unsigned char *buf, unsigned long int len)
 {
 	int i;
@@ -286,10 +284,10 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 		}
 		if ((len - bytes_read) > DATA_BUFFER) {
 			memcpy((char *)buffer, buf, xret);
-			(unsigned int)buffer += DATA_BUFFER;
+			buffer += DATA_BUFFER;
 		} else {
 			memcpy((char *)buffer, buf, (len - bytes_read));
-			(unsigned int)buffer += len - bytes_read;
+			buffer += len - bytes_read;
 		}
 
 		/* acknowledge the packet */
@@ -379,14 +377,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			}
 			exifdat.header = d;
 			exifdat.data = d+12;
-			thumbnail = exif_get_thumbnail_and_size(&exifdat,
+			thumbnail = gpi_exif_get_thumbnail_and_size(&exifdat,
 				&long_len);
 			free(d);
 			d = thumbnail;
-			if (ret < GP_OK) {
-				free(d);
-				return ret;
-			}
 			break;
 		case GP_FILE_TYPE_EXIF:
 			len = file_info.preview.size;
@@ -531,7 +525,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 			gp_context_progress_stop (context, id);
 			gp_context_error(context, 
 				_("Can't upload this image to the camera. "
-				"An error has occured."));
+				"An error has occurred."));
 			return (GP_ERROR);
 		}
 		state = NEXTFRAME;
@@ -596,7 +590,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	if (ack != ACK) {
 		gp_context_progress_stop (context, id);
 		gp_context_error(context, _("Can't upload this image to the "
-			"camera. An error has occured."));
+			"camera. An error has occurred."));
 		return (GP_ERROR);
 	}
 	gp_context_progress_stop (context, id);
@@ -638,13 +632,13 @@ camera_capture (Camera* camera, CameraCaptureType type, CameraFilePath* path,
 	if (ack == NACK) {
 		if (buf[CAMERA_MODE_PTR] != REC_MODE)
 			gp_context_error(context, _("You must be in record "
-				"mode to capture image."));
+				"mode to capture images."));
 		else if (!nbr_images)
 			gp_context_error(context, _("No space available "
-				"to capture new image. You must delete some "
+				"to capture new images. You must delete some "
 				"images."));
 		else
-			gp_context_error(context, _("Can't capture new image. "
+			gp_context_error(context, _("Can't capture new images. "
 				"Unknown error"));
 		return (GP_ERROR);
 	}
@@ -749,7 +743,7 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
 	 * but UNIX timestamp start the 1 January 1970 at 00:00
 	 * so we calculate the UNIX timestamp with the camera's one
 	 */
-	(int)timestamp += (8*365 + 2*366)*24*3600-3600;
+	timestamp += (8*365 + 2*366)*24*3600-3600;
 
 	/* Window creation */
 	gp_widget_new (GP_WIDGET_WINDOW, _("Konica Configuration"), window);
@@ -940,22 +934,22 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
        /* Focus */
         gp_widget_new (GP_WIDGET_RADIO, _("Focus"), &widget);
         gp_widget_append (section, widget);
-        gp_widget_add_choice (widget, _("2.0m"));
-	gp_widget_add_choice (widget, _("0.5m"));
-	gp_widget_add_choice (widget, _("0.1m"));
+        gp_widget_add_choice (widget, _("2.0 m"));
+	gp_widget_add_choice (widget, _("0.5 m"));
+	gp_widget_add_choice (widget, _("0.1 m"));
         gp_widget_add_choice (widget, _("Auto"));
         switch (buf[FOCUS_PTR]) {
 		case 0:
 			gp_widget_set_value (widget, _("Auto"));
 			break;
 		case 1:
-	                gp_widget_set_value (widget, _("2.0m"));
+	                gp_widget_set_value (widget, _("2.0 m"));
 	                break;
 	        case 2:
-        	        gp_widget_set_value (widget, _("0.5m"));
+        	        gp_widget_set_value (widget, _("0.5 m"));
                 	break;
 	        case 3:
-        	        gp_widget_set_value (widget, _("0.1m"));
+        	        gp_widget_set_value (widget, _("0.1 m"));
                 	break;
         }
 
@@ -963,14 +957,14 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         gp_widget_new (GP_WIDGET_RADIO, _("White balance"), &widget);
         gp_widget_append (section, widget);
         gp_widget_add_choice (widget, _("Office"));
-	gp_widget_add_choice (widget, _("Day-lt"));
+	gp_widget_add_choice (widget, _("Daylight"));
 	gp_widget_add_choice (widget, _("Auto"));
         switch (buf[WHITE_BALANCE_PTR]) {
 	        case 0:
         	        gp_widget_set_value (widget, _("Auto"));
                 	break;
 	        case 1:
-        	        gp_widget_set_value (widget, _("Day-lt"));
+        	        gp_widget_set_value (widget, _("Daylight"));
                 	break;
 	        case 2:
         	        gp_widget_set_value (widget, _("Office"));
@@ -1094,7 +1088,7 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         /* Self Timer */
         gp_widget_new (GP_WIDGET_RADIO, _("Self Timer"), &widget);
         gp_widget_append (section, widget);
-        gp_widget_add_choice (widget, _("Self Timer (only next picture)"));
+        gp_widget_add_choice (widget, _("Self Timer (next picture only)"));
         gp_widget_add_choice (widget, _("Normal"));
         switch (buf[TIMER_PTR]) {
 	        case 1:
@@ -1150,7 +1144,7 @@ camera_summary (Camera *camera, CameraText *text, GPContext *context)
 
 	timestamp = (buf[TIMESTAMP_PTR] << 24) + (buf[TIMESTAMP_PTR+1] << 16)
 		+ (buf[TIMESTAMP_PTR+2] << 8) + buf[TIMESTAMP_PTR+3];
-	(unsigned int)timestamp += (8*365 + 2*366)*24*3600-3600;
+	timestamp = (unsigned int)timestamp + (8*365 + 2*366)*24*3600-3600;
 	tmp = *localtime(&timestamp);
 	switch (buf[DATE_FORMAT_PTR]) {
 		case 1:
@@ -1159,7 +1153,7 @@ camera_summary (Camera *camera, CameraText *text, GPContext *context)
 			break;
 		case 2:
 			strftime(date,sizeof(date),"%Y/%m/%d %H:%M",&tmp);
-			snprintf(date_disp,sizeof(date_disp),_("YYYY/MM/AA"));
+			snprintf(date_disp,sizeof(date_disp),_("YYYY/MM/DD"));
 			break;
 		default:
 			strftime(date,sizeof(date),"%m/%d/%Y %H:%M",&tmp);
@@ -1168,9 +1162,9 @@ camera_summary (Camera *camera, CameraText *text, GPContext *context)
 	}
 	snprintf(text->text, sizeof(text->text), 
 			_("Model: %s\n"
-			"Capacity: %iMo\n"
+			"Capacity: %i Mb\n"
 			"Power: %s\n"
-			"Auto Off Time: %imin\n"
+			"Auto Off Time: %i min\n"
 			"Mode: %s\n"
 			"Images: %i/%i\n"
 			"Date display: %s\n"
@@ -1205,14 +1199,13 @@ static int
 camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	snprintf(manual->text,sizeof(manual->text),
-		_("About Konica Q-M150:\n"
-		"This camera does not support to do any changes\n"
-		"from the outside. So in the configuration, you could\n"
-		"just see what it is configured on the camera\n"
-		"but you can't change anything.\n\n"
-		"If you have some issues with this driver, please\n"
-		"send an e-mail to the authors of this driver.\n\n"
-		"Thanks to use GPhoto2 and UNIX/Linux Operating systems.\n"));
+	_("About Konica Q-M150:\n"
+	"This camera does not allow any changes\n"
+	"from the outside. So in the configuration, you can\n"
+	"only see what it is configured on the camera\n"
+	"but you can not change anything.\n\n"
+	"If you have some issues with this driver, please e-mail its authors.\n"
+	));
 	return (GP_OK);
 }
 
@@ -1254,6 +1247,15 @@ camera_abilities (CameraAbilitiesList *list)
 /*
  * Initialization of the camera
  */
+
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -1272,12 +1274,7 @@ camera_init (Camera *camera, GPContext *context)
 
 
 	/* Now, tell the filesystem where to get lists, files and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func, 
-		delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, put_file_func, 
-		delete_all_func, NULL, NULL, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
 	/*
 	 * The port is already provided with camera->port (and
