@@ -30,8 +30,8 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <gphoto2.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2.h>
+#include <gphoto2/gphoto2-port.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -78,7 +78,7 @@ static int cam_has_flash (CameraPrivateLibrary *pl);
 static int cam_has_card (CameraPrivateLibrary *pl);
 
 /* define what cameras we support */
-static struct
+static const struct
 {
 	char *model;
 	int usb_vendor;
@@ -99,6 +99,9 @@ models[] =
 		BRIDGE_SPCA504, SPCA50X_SDRAM },
 	{"Aiptek:Pencam", 0x04fc, 0x504a,
 		BRIDGE_SPCA504, SPCA50X_SDRAM | SPCA50X_FLASH },
+	/* same ids, different cam *sigh* */
+	{"Aiptek:Pencam without flash", 0x04fc, 0x504a,
+		BRIDGE_SPCA504, SPCA50X_SDRAM },
 	{"Medion:MD 5319", 0x04fc, 0x504a,
 		BRIDGE_SPCA504, SPCA50X_SDRAM | SPCA50X_FLASH },
 	{"nisis:Quickpix Qp3", 0x04fc, 0x504a,
@@ -120,7 +123,9 @@ models[] =
 	{"D-Link:DSC 350+", 0x084d, 0x0003,
 		BRIDGE_SPCA500, SPCA50X_FLASH},
         {"Minton:S-Cam F5", 0x084d, 0x0003,
-	        BRIDGE_SPCA500, SPCA50X_FLASH},
+	        BRIDGE_SPCA500, SPCA50X_FLASH},	
+	{"PureDigital:Ritz Disposable", 0x04fc, 0xffff,
+		BRIDGE_SPCA504B_PD, SPCA50X_FLASH},
 	{NULL, 0, 0, 0, 0}
 };
 
@@ -160,6 +165,9 @@ camera_abilities (CameraAbilitiesList *list)
 			 || a.usb_product == 0xc520)
 				a.operations = GP_OPERATION_CAPTURE_IMAGE;
 		}
+		if (models[x].bridge == BRIDGE_SPCA504B_PD) {
+                            a.operations = GP_OPERATION_CAPTURE_IMAGE;
+                }
 		if (models[x].bridge == BRIDGE_SPCA500) {
 			/* TEST enable capture for the DSC-350 style cams */
 			if (a.usb_vendor == 0x084d) {
@@ -279,14 +287,6 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 	return (GP_OK);
 }
 
-static int
-camera_manual (Camera *camera, CameraText *about, GPContext *context)
-{
-	return (GP_OK);
-}
-
-
-
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
@@ -494,6 +494,9 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	if (cam_has_flash(camera->pl) || cam_has_card(camera->pl) ) {
 		CHECK (spca50x_flash_get_filecount
 					(camera->pl, &flash_file_count));
+	} else {
+		/* should not happen really */
+		return GP_ERROR;
 	}
 	if (n < flash_file_count) {
 		return spca500_flash_delete_file (camera->pl, n);
@@ -505,7 +508,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 
 		gp_filesystem_name (fs, "/", c - 1, &name, context);
 		gp_context_error (context,
-				  _("Your camera does only support deleting the "
+				  _("Your camera only supports deleting the "
 				   "last file on the camera. In this case, this "
 				   "is file '%s'."), name);
 		return (GP_ERROR);
@@ -528,10 +531,18 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	return GP_OK;
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.get_info_func = get_info_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
-	int ret;
+	int ret = 0;
 	int x = 0;
 	char *model;
 
@@ -541,7 +552,6 @@ camera_init (Camera *camera, GPContext *context)
 	/* First, set up all the function pointers */
 	camera->functions->exit = camera_exit;
 	camera->functions->summary = camera_summary;
-	camera->functions->manual = camera_manual;
 	camera->functions->about = camera_about;
 	camera->functions->capture = camera_capture;
 
@@ -582,10 +592,19 @@ camera_init (Camera *camera, GPContext *context)
 	while (model) {
 		if (abilities.usb_vendor == models[x].usb_vendor
 		 && abilities.usb_product == models[x].usb_product) {
-			camera->pl->bridge = models[x].bridge;
-			camera->pl->storage_media_mask =
-				models[x].storage_media_mask;
-			break;
+			int same;
+			char *m = strdup( models[x].model );
+			char *p = strchr (m, ':' );
+
+			if (p) *p = ' ';
+			same = !strcmp (m, abilities.model);
+			free (m);
+			if (same) {
+				camera->pl->bridge = models[x].bridge;
+				camera->pl->storage_media_mask =
+					models[x].storage_media_mask;
+				break;
+			}
 		}
 		model = models[++x].model;
 	}
@@ -594,12 +613,20 @@ camera_init (Camera *camera, GPContext *context)
 	if (camera->pl->fw_rev > 1) {
 		CHECK (spca50x_detect_storage_type (camera->pl));
 	}
+   
 	if (cam_has_flash(camera->pl) || cam_has_card(camera->pl) ) {
-		if (camera->pl->bridge == BRIDGE_SPCA504)
+		if ((camera->pl->bridge == BRIDGE_SPCA504) ||
+		    (camera->pl->bridge == BRIDGE_SPCA504B_PD))
 			CHECK (spca50x_flash_init (camera->pl, context));
 	}
 
-	ret = spca50x_reset (camera->pl);
+	if ((camera->pl->bridge == BRIDGE_SPCA504) ||
+	    (camera->pl->bridge == BRIDGE_SPCA504B_PD)) {
+/*		if (abilities.usb_vendor != 0x04fc && abilities.usb_product != 0x504a ) */
+		if (!(abilities.usb_vendor == 0x04fc && abilities.usb_product == 0x504a ))
+			ret = spca50x_reset (camera->pl);
+	}
+
 	if (ret < 0) {
 		gp_context_error (context, _("Could not reset camera.\n"));
 		free (camera->pl);
@@ -609,16 +636,7 @@ camera_init (Camera *camera, GPContext *context)
 	}
 
 	/* Set up the CameraFilesystem */
-	CHECK (gp_filesystem_set_list_funcs
-	       (camera->fs, file_list_func, NULL, camera));
-	CHECK (gp_filesystem_set_file_funcs
-	       (camera->fs, get_file_func, delete_file_func, camera));
-	CHECK (gp_filesystem_set_info_funcs
-	       (camera->fs, get_info_func, NULL, camera));
-	CHECK (gp_filesystem_set_folder_funcs
-	       (camera->fs, NULL, delete_all_func, NULL, NULL, camera));
-
-	return (GP_OK);
+	return gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 }
 
 static int
@@ -630,13 +648,11 @@ cam_has_sdram (CameraPrivateLibrary *pl)
 static int
 cam_has_flash (CameraPrivateLibrary *pl)
 {
-
 	return pl->storage_media_mask & SPCA50X_FLASH;
 }
 
 static int
 cam_has_card (CameraPrivateLibrary *pl)
 {
-
 	return pl->storage_media_mask & SPCA50X_CARD;
 }
